# FIND COUNTS OF FOR HOW MANY CASES EACH VARIABLE IS NON-ZERO.  Does this for
# training cases (from train.x, by default) and for all cases (also from valid.x
# and test.x, by default).  For training cases, overall counts (train.n)
# and counts for just class 0 (train.n0) and just class 1 (train.n1) are found.
# Also creates a new set of variables (train.cnt, valid.cnt, test.cnt, all.cnt)
# holding the number of variables that are nonzero once, twice, etc. (up to k)
# in the whole data set that are non-zero in each case.  Everything is returned
# in these global variables, nothing as the value of this function.

count.nonzero <- function (trn.x=(train.x!=0), vld.x=(valid.x!=0), 
                           tst.x=(test.x!=0), k=2)
{
  # Count the number of times each variable is non-zero.  Unfortunately,
  # using "apply" to do this uses up more memory.

  train.n <<- 0
  for (i in 1:nrow(trn.x))
  { train.n <<- train.n + trn.x[i,]
  }
  
  all.n <<- train.n
  for (i in 1:nrow(vld.x))
  { all.n <<- all.n + vld.x[i,]
  }
  for (i in 1:nrow(tst.x))
  { all.n <<- all.n + tst.x[i,]
  }

  # Count the number of times each variable is non-zero in training cases
  # where the class is 0 or 1.

  train.n0 <<- 0
  train.n1 <<- 0
 
  for (i in 1:nrow(trn.x))
  { if (train.y[i]==0)
    { train.n0 <<- train.n0 + trn.x[i,]
    }
    else
    { train.n1 <<- train.n1 + trn.x[i,]
    }
  }

  # Create new variables, with the value of new variable i for case c being 
  # the count how many variables that are non-zero in exactly i cases are
  # non-zero for case c.

  train.cnt <<- matrix(0,nrow(trn.x),k)
  valid.cnt <<- matrix(0,nrow(vld.x),k)
  test.cnt  <<- matrix(0,nrow(tst.x),k)

  for (i in 1:k)
  { s <- sum(all.n==i)
    if (s==1)
    { train.cnt[,i] <<- as.vector(trn.x[,all.n==i])
      valid.cnt[,i] <<- as.vector(vld.x[,all.n==i])
      test.cnt[,i]  <<- as.vector(tst.x[,all.n==i])
    }
    else if (s!=0)
    { train.cnt[,i] <<- apply(trn.x[,all.n==i],1,sum)
      valid.cnt[,i] <<- apply(vld.x[,all.n==i],1,sum)
      test.cnt[,i]  <<- apply(tst.x[,all.n==i],1,sum)
    }
  }

  all.cnt <<- rbind(train.cnt,valid.cnt,test.cnt)

  invisible()
}
