# PROCESS MADELON DATA - STAGE 2.

source("../../select.r")
source("../../pca.r")
source("../../plot.r")

# Find pvalues for tests of association of features with class. 

pval.perm <- perm.pval(train.x,ranks=TRUE)
pval.runs <- runs.pval(train.x)   # TAKES ABOUT 20 MINUTES

# Produce various plots of p-values.

postscript("madelon2a.ps",horizontal=TRUE)

par(mfrow=c(1,3))

minp <- 0.5/ncol(train.x)

plot (pmax(minp,sort(pval.perm)), pch=20, log="y", ylab="",
      xlab="Permutation test p-values", ylim=c(minp,1))
lines(1:ncol(train.x),((1:ncol(train.x))-0.5)/ncol(train.x))
abline(v=c(0,50,100),lty=3)

plot (pmax(minp,sort(pval.runs)), pch=20, log="y", ylab="", 
      xlab="Runs test p-values", ylim=c(minp,1))
lines(1:ncol(train.x),((1:ncol(train.x))-0.5)/ncol(train.x))
abline(v=c(0,50,100),lty=3)

jitter <- exp(rnorm(ncol(train.x),0,0.05))

plot (pval.runs*jitter, pval.perm*jitter, log="xy", pch=20,
      xlab="Runs test p-values", ylab="Permutation test p-values")
abline(h=quantile(pval.perm,c(0.01,0.1)))
abline(v=quantile(pval.runs,c(0.01,0.1)))

dev.off()

# Select a set of features using these pvalues.

ix.sel <- order(pval.perm)[1:38]   # p-value <= 0.05

write.ix(ix.sel,"sel")
write.data(ix.sel,"sel")

# Find the first 40 principle components from the features.

pca <- pca.vectors (all.x, 40, center=TRUE, scale=TRUE) 
write.pca (pca, "pc", train.x, valid.x, test.x)
